-- Graph Library
-- Gordon Pace - Gerardo Schneider - VERIMAG 
-- 16/07/2001

module Graph (
  Graph, graphTMAF, makeGraph, graphDomain, edgeRange,
  module Input
)  where

import Array
import AffineFunctions
import Input
import Maybe
import Interval

data Graph = 
  Graph {
    transitionFunction :: EdgeId -> EdgeId -> Maybe TMAF,
    domain :: [EdgeId],
    edgeRange :: EdgeId -> Interval RealPlus
  }

instance Show Graph where
  show graph =
     unlines $ 
        ("Domain: "++show dom):
        map (\(s,d,Just x) -> "("++show s++", "++show d++")\n  "++show x) 
        (filter (\(_,_,x) -> isJust x) [ (src, dst, trans src dst) | src <- dom, dst <- dom ])
    where
      dom = domain graph
      trans = transitionFunction graph
   
-- Given a graph and two nodes (source edge id, destination edge id)
-- return the one-step TMAF (if one exists)
graphTMAF :: Graph -> EdgeId -> EdgeId -> Maybe TMAF
graphTMAF = transitionFunction

graphDomain :: Graph -> [EdgeId]
graphDomain = domain

-- Covert Input to (efficient retreival) Graph format
makeGraph :: Input -> Graph
makeGraph input = 
  Graph { 
    transitionFunction = \e1 e2 -> matrix ! (e1,e2),
    domain = [0..ecount-1],
    edgeRange = lambdaRange . getEdge input 
  }
  where
    ecount = countEdges input

    matrix = array ((0,0), (ecount-1,ecount-1)) 
      [ ((e1, e2), edgeToEdgeTMAF input e1 e2) 
      | e1 <- [0..ecount-1]
      , e2 <- [0..ecount-1]
      ]
       
