module LoopAnalysis (
  getEventHorizons, loopsOf
) where

import List
import Maybe

import Infinity
import Interval

import Graph
import Signature
import Successor
import Generator

realline = toInterval (closed InfiNeg, closed InfiPos) :: Interval RealPlus
 
-- Calculate the event horizon of all stay loops
getEventHorizons :: Graph -> [[(EdgeId, Interval RealPlus)]]
getEventHorizons graph = 
  map (getEventHorizonOfLoop graph) loops
  where
    domain      = graphDomain graph
    transitions = graphToRelation graph
    loops       = filter (\path->isStay (fromJust (loopType graph (Path path)))) $
                  filter (\path->feasibleLoop graph (head path) (Path path)) $
                  loopsOf graph domain transitions

-- Calculate the event horizon of a stay loop
getEventHorizonOfLoop :: Graph -> [EdgeId] -> [(EdgeId, Interval RealPlus)]
getEventHorizonOfLoop graph edges = 
  zip edges intervals
  where
    tmafs     = [ fromJust (graphTMAF graph e e')
                | e:e':_ <- tails (edges ++ [head edges])
                ] 
    forward   = scanl comp funcId tmafs
    backward  = scanr comp funcId tmafs

    intervals = [ let tmaf = comp tmaf2 tmaf1 in fromTMAF2S tmaf /\ fromTMAF2J tmaf
                | (tmaf1, tmaf2) <- zip backward forward
                ]


-- are two paths equivalent but possible rotated? We assume that the paths contain no duplicates
x === y =
  let (y_before, y_after) = break (head x ==) y
  in  y_after++y_before == x

-- TODO Needs to be rewritten efficiently
loopsOf graph nodes transitions =
  nubBy (===) $ concat $ map (loopsOf' funcId []) nodes
  where
    loopsOf' tmaf history node 
      | isEmpty (apply tmaf realline) = []
      | null loop = concat [ loopsOf' (comp tmaf (fromJust $ graphTMAF graph node node')) history' node' 
                           | (node0,node') <- transitions
                           , node0==node
                           ]
      | otherwise = [loop]
      where
        history' = history ++ [node]
        (before, loop) = break (node ==) history

