
import Char
import List
import Maybe
import System
import IO

import Version
import Useful
import Parser
import AffineFunctions
import Input
import UserInput
import FileHandling
import UserInput

import Graph
import Fig
import Interval
import Signature
import Successor
import ConcretePath
import Generator

import Kernel

program_name = "drawkernels"
program_description = "Draw the kernel of a given spdi"++version
program_usage =
    "Usage: "++program_name++" <file.spdi> [<file.fig>]\n"++
    "  where:\n"++
    "    <file.spdi> is the name of the input file\n"

parse_arguments = 
  \ ~(spdi, _) -> ( parseFilenameExtension "spdi"
                  , parseFilenameExtension "fig" `defaultsTo` changeFileExtension spdi "spdi" "fig"
                  )

main =
  do { (spdi,fig) <- runProgramFix (program_description, program_usage) parse_arguments
     ; uinput <- fileToUserInput spdi
     ; (input,point2vector,points2edgeID,edgeID2points) <- fileToInput spdi 
     ; let graph = makeGraph input
     ; let loops = allSimpleLoops graph
     ; putStrLn (show (length loops) ++" loop/s found.")
     ; let t = fromJust . interval2extremes input
     ; figMultipleKernelFile uinput fig 
         ( -- areas
         zip3 via_dpts (repeat via_col) 
         [ getArea input (viability graph loop)
         | loop <- loops
         , isJust $ loopType graph loop
         , not $ isDie $ fromJust $ loopType graph loop 
         ] -- viability kernels 
         ++ 
         zip3 inv_dpts (repeat inv_col) 
         [ getArea input (invariance graph loop)
         | loop <- loops
         , isJust $ loopType graph loop
         , isStay $ fromJust $ loopType graph loop 
         ] -- invariance kernels 
         )
         ( -- intersections
         zip3 cnt_dpts (repeat cnt_col) 
         [ [ ((t a, t b),(t c, t d)) | ((a,b),(c,d)) <- controllability graph loop ]
         | loop <- loops
         , isJust $ loopType graph loop
         , not $ isDie $ fromJust $ loopType graph loop 
         ] -- controllability kernels
         )
     ; putStrLn ""
     ; putStrLn "Invariance kernels      => BLUE"
     ; putStrLn "Controllability kernels => GREEN"
     ; putStrLn "Viability kernels       => YELLOW"
     } `catch` showError
    


inv_col = 1 -- Blue
cnt_col = 2 -- Green
via_col = 6 -- Yellow

cnt_dpt = 100
inv_dpt = 200
via_dpt = 300

inv_dpts = from inv_dpt
cnt_dpts = from cnt_dpt
via_dpts = from via_dpt

from n = [ n+i | i <- [0,1..] ]
