
import Char
import Maybe
import List
import System

import Version
import Useful
import Parser

import AffineFunctions
import Signature 

import Input
import FileHandling
import UserInput

program_name = "getmafs"
program_description = "Show MAFs from SPDI file "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> [-p <pathfile> | <edgelist>]\n"++
    "  where <edgelist> is a comma separated list of edges,\n"++
    "  each edge being specified as two point labels separated\n"++
    "  by a dash eg 1-2,4-12,origin-3.\n"++
    "  <pathfile> is a filename (default extension .path) which\n"++
    "  contains an edgelist with the same syntax as above.\n"

parse_arguments = ( parseFilenameExtension "spdi"
                  , parseOption1Parameter "p" (parseFilenameExtension "path") <+> parseEdges
                  ) 

showMAFs trans_mafs = 
    unlines $ zipWith showTransitionMAF trans_mafs cmafs
  where
    showTransitionMAF ((edge1,edge2),maf) cmaf = 
      "From edge "++edge1++" to edge "++edge2++": MAF is "++ show maf ++ 
      "\n   (accumulated MAF is "++show cmaf++")"
    cmafs = scanl1 comp (map snd trans_mafs)
    

path2mafs :: (EdgeId -> (String, String)) -> Input -> Path -> IO [((EdgeId, EdgeId), MAF)]
path2mafs edgeID2points input path
  | any (isNothing . snd) maybe_mafs = 
       fail $ "There is no transitions between the edges "++
              showEdge edgeID2points src++
              " and "++
              showEdge edgeID2points dst
  | otherwise                 = return mafs
  where
    maybe_mafs    = [((src,dst), edgeToEdgeMAF input src dst) | src:dst:_ <- tails (fromPath path)]
    ((src,dst),_) = head $ filter (isNothing . snd) maybe_mafs
    mafs          = [(trans, maf) | (trans, Just maf) <- maybe_mafs]

main =
  do { (filename, edgelist_desc) <- runProgram (program_description, program_usage) parse_arguments
     ; edgelist <- case edgelist_desc of 
                     (Just fn, Nothing) -> parseEdgesFromFile fn
                     (Nothing, Just es) -> return es
     ; (input, point2vector, points2edgeID, edgeID2points) <- fileToInput filename
     ; path <- readPath (point2vector,points2edgeID) edgelist
     ; transitions_mafs <- path2mafs edgeID2points input path
     ; putStrLn "The requested MAFs:"
     ; putStrLn $
         showMAFs [ ((showEdge edgeID2points src, showEdge edgeID2points dst), maf)
                  | ((src,dst), maf) <- transitions_mafs
                  ]
  } `catch` showError

