
import Char
import Maybe
import List
import System

import Version
import Useful
import Parser

import AffineFunctions
import Signature 

import Input
import FileHandling
import UserInput

program_name = "gettmafs"
program_description = "Show TMAFs from SPDI file "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> [-p <pathfile> | <edgelist>]\n"++
    "  where <edgelist> is a comma separated list of edges,\n"++
    "  each edge being specified as two point labels separated\n"++
    "  by a dash eg 1-2,4-12,origin-3.\n"++
    "  <pathfile> is a filename (default extension .path) which\n"++
    "  contains an edgelist with the same syntax as above.\n"

parse_arguments = ( parseFilenameExtension "spdi"
                  , parseOption1Parameter "p" (parseFilenameExtension "path") <+> parseEdges
                  ) 

showTMAFs trans_tmafs = 
    unlines $ zipWith showTransitionTMAF trans_tmafs ctmafs
  where
    showTransitionTMAF ((edge1,edge2),tmaf) ctmaf = 
      "From edge "++edge1++" to edge "++edge2++": TMAF is "++ show tmaf ++ 
      "\n   (accumulated TMAF is "++show ctmaf++")"
    ctmafs = scanl1 comp (map snd trans_tmafs)
    

path2tmafs :: (EdgeId -> (String, String)) -> Input -> Path -> IO [((EdgeId, EdgeId), TMAF)]
path2tmafs edgeID2points input path
  | any (isNothing . snd) maybe_tmafs = 
       fail $ "There is no transitions between the edges "++
              showEdge edgeID2points src++
              " and "++
              showEdge edgeID2points dst
  | otherwise                 = return tmafs
  where
    maybe_tmafs   = [((src,dst), edgeToEdgeTMAF input src dst) | src:dst:_ <- tails (fromPath path)]
    ((src,dst),_) = head $ filter (isNothing . snd) maybe_tmafs
    tmafs         = [(trans, tmaf) | (trans, Just tmaf) <- maybe_tmafs]

main =
  do { (filename, edgelist_desc) <- runProgram (program_description, program_usage) parse_arguments
     ; edgelist <- case edgelist_desc of 
                     (Just fn, Nothing) -> parseEdgesFromFile fn
                     (Nothing, Just es) -> return es
     ; (input, point2vector, points2edgeID, edgeID2points) <- fileToInput filename
     ; path <- readPath (point2vector,points2edgeID) edgelist
     ; transitions_tmafs <- path2tmafs edgeID2points input path
     ; putStrLn "The requested TMAFs:"
     ; putStrLn $
         showTMAFs [ ((showEdge edgeID2points src, showEdge edgeID2points dst), tmaf)
                   | ((src,dst), tmaf) <- transitions_tmafs
                   ]
  } `catch` showError

