
import Char
import List
import Maybe
import System
import IO

import Version
import Useful
import Parser
import AffineFunctions
import Input
import UserInput
import FileHandling
import UserInput

import Graph
import Fig
import Interval
import Signature
import Successor
import ConcretePath

import Kernel

program_name = "invariance"
program_description = "Calculate the invariance kernel of a given STAY loop "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> [-p <pathfile> | <edgelist>]\n"++
    "  where:\n"++
    "    <file.spdi> is the name of the input file\n"++
    "    <edgelist> is a comma separated list of edges, each edge being specified as two\n"++
    "      point labels separated by a dash eg 1-2,4-12,origin-3.\n"++
    "    <pathfile> is a filename (default extension .path) which contains an edgelist\n"++
    "      with the same syntax as above.\n"

parse_arguments = ( parseFilenameExtension "spdi"
                  , parseOption1Parameter "p" (parseFilenameExtension "path") <+> parseEdges
                  )

main =
  do { (spdi, edges_desc) <- 
           runProgram (program_description, program_usage) parse_arguments
     ; edges <- case edges_desc of 
                     (Just fn, Nothing) -> parseEdgesFromFile fn
                     (Nothing, Just es) -> return es
     ; uinput <- fileToUserInput spdi
     ; (input,point2vector,points2edgeID,edgeID2points) <- fileToInput spdi 
     ; path <- readPath (point2vector, points2edgeID) (edges::[(String, String)])
     ; let graph = makeGraph input

     ; putStrLn "\nPath given:"
     ; putStrLn $ unlines 
          [showEdge edgeID2points eid | eid <- fromPath path]

     ; case loopType graph path of
         Nothing -> fail "Loop not possible"
         Just (Stay _)-> 
           do { let inv = invariance graph path
              ; putStrLn (unlines $ map show inv)
              ; let kernel = getArea input inv  
              ; figKernelFile uinput "path.fig" kernel 
              }
         Just _ -> fail "No viability kernel can be calculated for non-STAY loops"
     } `catch` showError



{-- 


TO FIX:

1. two edges not adjacent - wrong error

--}

