import Char
import Maybe
import List
import System

import Version
import Useful
import Parser

import AffineFunctions
import Input
import Graph
import FileHandling
import UserInput
import Signature
import Successor

program_name = "looptype"
program_description = "Show loop type in an SPDI "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> [-p <pathfile> | <edgelist>]\n"++
    "  where <edgelist> is a comma separated list of edges,\n"++
    "  each edge being specified as two point labels separated\n"++
    "  by a dash eg 1-2,4-12,origin-3.\n"++
    "  <pathfile> is a filename (default extension .path) which\n"++
    "  contains an edgelist with the same syntax as above.\n"

parse_arguments = ( parseFilenameExtension "spdi"
                  , parseOption1Parameter "p" (parseFilenameExtension "path") <+> parseEdges
                  ) 

main =
  do { (filename, edgelist_desc) <- runProgram (program_description, program_usage) parse_arguments
     ; edgelist <- case edgelist_desc of 
                     (Just fn, Nothing) -> parseEdgesFromFile fn
                     (Nothing, Just es) -> return es
     ; (input, point2vector, points2edgeID,edgeID2points) <- fileToInput filename
     ; path <- readPath (point2vector,points2edgeID) edgelist
     ; case loopType (makeGraph input) path of
         Nothing -> fail "Loop not possible"
         Just lp -> putStrLn ("Loop type:\n   "++show lp)
     } `catch` showError 

