
import Maybe
import System
import List
import Char

import Version
import Useful
import Parser

import FileHandling
import UserInput
import Input

import Vector
import Interval
import AffineFunctions
import Signature
import ConcretePath

import Graph
import Fig

program_name = "path2fig"
program_description = "Produce a fig file showing an path "++version 
program_usage =
    "Usage: "++program_name++" [-over|-under] [-o <spdi.fig>] <file.spdi> \n"++
    "                 <input interval> [-p <pathfile> | <edgelist>]\n"++
    "  where:\n"++
    "    -over and -under regulates the relative depth of the areas in the signature.\n"++
    "      -under is the default, and later traversed areas appear underneath older\n"++
    "      ones - this is usually best to generate a fixed fig image. -over does the\n"++
    "      opposite, and is usually best if the fig is intended to generate an animated gif.\n"++
    "    <file.fig> is the name of the fig file to generate.\n"++
    "    <file.spdi> is the name of the input file.\n"++
    "    <edgelist> is a comma separated list of edges, each edge being specified as two\n"++
    "      point labels separated by a dash eg 1-2,4-12,origin-3.\n"++
    "    <pathfile> is a filename (default extension .path) which contains an edgelist\n"++
    "      with the same syntax as above.\n"++
    "    <input interval> is the input interval written as a pair of\n"++
    "      comma separated numbers within square brackets (eg [0.5,1.2]).\n"

parse_arguments =
   \ ~(_,_,spdi,_,_) ->
     ( applyParser ("under"==) (parseOption "over" <|> parseOption "under") `defaultsTo` True
     , parseOption1Parameter "o" parseFilename `defaultsTo` changeFileExtension spdi "spdi" "fig"
     , parseFilenameExtension "spdi" 
     , parseInterval 
     , parseOption1Parameter "p" (parseFilenameExtension "path") <+> parseEdges
     ) 

path2mafs :: (EdgeId -> (String, String)) -> Input -> Path -> IO [((EdgeId, EdgeId), MAF)]
path2mafs edgeID2points input path
  | any (isNothing . snd) maybe_mafs =
       fail $ "There is no transitions between the edges "++
              showEdge edgeID2points src++
              " and "++
              showEdge edgeID2points dst
  | otherwise                 = return mafs
  where
    maybe_mafs    = [((src,dst), edgeToEdgeMAF input src dst) | src:dst:_ <- tails (fromPath path)]
    ((src,dst),_) = head $ filter (isNothing . snd) maybe_mafs
    mafs          = [(trans, maf) | (trans, Just maf) <- maybe_mafs]


main =
  do { (deeper,fig,spdi,interval,edges_desc) <- runProgramFix (program_description, program_usage) parse_arguments
     ; edges <- case edges_desc of 
                     (Just fn, Nothing) -> parseEdgesFromFile fn
                     (Nothing, Just es) -> return es
     ; userinput <- fileToUserInput spdi
     ; let point2vector str = lookup str (zip (pointNames userinput) (vertexTable userinput))
     ; let (input,points2edgeID,edgeID2points) = userInputToInput userinput
     ; path <- readPath (point2vector, points2edgeID) (edges::[(String, String)])
     ; path2mafs edgeID2points input path
     ; let graph = makeGraph input
     ; let transitions = [ (src,dst) | src:dst:_ <- tails (fromPath path) ]
     ; let tmafs = map (fromJust . uncurry (graphTMAF graph)) transitions
     ; let intervals = scanl (\interval tmaf -> apply tmaf interval) interval tmafs
     ; case getTrace input (zip (fromPath path) intervals) of
         Nothing     -> fail "The given path is not feasable"
         Just result -> figPathFile userinput fig result deeper
     } `catch` showError

