
import Char
import Maybe
import List
import System
import IO

import Version
import Useful
import Parser
import AffineFunctions
import Input
import FileHandling
import UserInput

import Graph
import Interval
import Signature
import Generator
import Successor

program_name = "reachable"
program_description = "Show feasability of going from one edge to another in an SPDI "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> <input interval> <exit interval> <source> <destination>\n"++
    "  where:\n"++
    "    <file.spdi> is the name of the input file\n"++
    "    <input interval> is the input interval written as a pair of\n"++
    "      comma separated numbers within square brackets (eg [0.5,1.2])\n"++
    "    <exit interval> is the exiting intercal written in the same notation\n"++
    "    <source> and <destination> are both edges specified as two point\n"++
    "      labels separated by a dash (eg 1-2)"
parse_arguments = 
    (parseFilenameExtension "spdi", (parseInterval, parseInterval), (parseEdge, parseEdge))

main =
  do { (spdi, (interval_0, interval_f), (src, dst)) <- 
          runProgram (program_description, program_usage) parse_arguments
     ; (input, point2vector, points2edgeID,edgeID2points) <- fileToInput spdi
     ; src' <- readUnambiguousEdge (point2vector, points2edgeID) src
     ; dst' <- readUnambiguousEdge (point2vector, points2edgeID) dst
     ; putStrLn ("Generating and trying signatures from edge "++
         showStringEdge src++" to "++
         showStringEdge dst++" ...")
     ; putStrLn ("Starting interval: "++show interval_0)
     ; putStrLn ("Finishing interval: "++show interval_f)
     ; let graph = makeGraph input
     ; let (transitions,signatures,loops) = generateSignatures graph src' dst'
     ; putStrLn ("Resulting reduced graph has "++show (length transitions)++" transitions.")
     ; let results = takeUntil snd (reach graph interval_0 interval_f signatures loops)
     ; let reachable = any snd results
     ; if results == [] then putStrLn "No signatures to explore" else
          putStrLn $ enumerateShow 
            (\(sig,sol)->showSignature edgeID2points sig++if sol then "<reachable>" else "<unreachable>") results
     ; putStrLn ("\n" ++ if reachable then "REACHABLE" else "UNREACHABLE")
     } `catch` showError


