import Maybe
import System
import IO

import Version
import Useful
import Parser
import FileHandling
import UserInput
import Fig

program_name = "sig2fig"
program_description = "Signature-To-Fig utility "++version
program_usage = 
    "Usage: "++program_name++" [-o <spdi.fig>] <file.spdi> [-s <sigfile> | <signature>]\n" ++
    "  where:\n"++
    "    <file.fig> is the name of the fig file to generate\n"++
    "    <file.spdi> is the name of the input file\n"++
    "    <signature> is a comma separated list of edges,\n"++
    "      marking the loops by enclosing them in square brackets.\n"++
    "      Each edge is specified as two point labels separated\n"++
    "      by a dash (eg 1-2 [1-2,4-12,origin-3] 2-3,4-6)\n"++
    "    <sigfile> is a filename (default extension .sig) which\n"++
    "      contains a signature with the same syntax as above.\n"

parse_arguments = 
  \ ~(_,spdi,_) -> 
     ( parseOption1Parameter "o" (parseFilenameExtension "fig") `defaultsTo` 
       changeFileExtension spdi "spdi" "fig"
     , parseFilenameExtension "spdi"
     , parseOption1Parameter "s" (parseFilenameExtension "sig") <+> parseSignature
     )

main = 
  do { (fig, spdi, signature_desc) <- 
              runProgramFix (program_description, program_usage) parse_arguments
     ; signature <- case signature_desc of 
                     (Just fn, Nothing) -> parseSignatureFromFile fn
                     (Nothing, Just sig) -> return sig
     ; uinput <- fileToUserInput spdi
     ; let point2vector point = lookup point $ zip (pointNames uinput) (vertexTable uinput)
     ; signature'  <- readVectors point2vector (signature :: [[(String,String)]])
     ; figSignatureFile uinput fig (map (map (\(v1,v2) -> (v1+v2) / 2)) signature') True 
     } `catch` showError

