
import Char
import List
import Maybe
import System
import IO

import Version
import Useful
import Parser
import AffineFunctions
import Input
import FileHandling
import UserInput

import Graph
import Interval
import Signature
import Successor
import ConcretePath

program_name = "simsig"
program_description = "Generate a concrete trace from an abstract signature "++version
program_usage =
    "Usage: "++program_name++" <file.spdi> <input interval> <exit interval> [-s <sigfile> | <signature>]\n"++
    "  where:\n"++
    "    <file.spdi> is the name of the input file\n"++
    "    <input interval> is the input interval written as a pair of\n"++
    "      comma separated numbers within square brackets (eg [0.5,1.2])\n"++
    "    <exit interval> is the exiting intercal written in the same notation\n"++
    "    <signature> is a comma separated list of edges,\n"++
    "      marking the loops by enclosing them in square brackets.\n"++
    "      Each edge is specified as two point labels separated\n"++
    "      by a dash (eg 1-2 [1-2,4-12,origin-3] 2-3,4-6).\n"++
    "    <sigfile> is a filename (default extension .sig) which\n"++
    "      contains a signature with the same syntax as above.\n"

parse_arguments = ( parseFilenameExtension "spdi"
                  , (parseInterval, parseInterval)
                  , parseOption1Parameter "s" (parseFilenameExtension "sig") <+> parseSignature
                  )

main =
  do { (spdi, (interval_0, interval_f), signature_desc) <- 
           runProgram (program_description, program_usage) parse_arguments
     ; signature <- case signature_desc of 
                     (Just fn, Nothing) -> parseSignatureFromFile fn
                     (Nothing, Just sig) -> return sig
     ; (input,point2vector,points2edgeID,edgeID2points) <- fileToInput spdi 
     ; signature' <- readSignature (point2vector, points2edgeID) signature

     ; putStrLn (showSignature edgeID2points signature')
     ; putStrLn ("Starting interval: "++show interval_0)
     ; putStrLn ("Finishing interval: "++show interval_f)

     ; let graph = makeGraph input
     ; let concrete_path = concretiseSignature graph signature' interval_0 interval_f
     ; path <- extractFromMaybe "\nNo concrete instance of signature found" concrete_path
     ; putStrLn "\nConcrete instance of signature:"
     ; putStrLn $ unlines 
          [showEdge edgeID2points eid ++" "++show interval | (eid, interval) <- path]
     } `catch` showError

